# -*- coding: utf-8 -*-
# Description: rrdcached netdata python.d module
# Author: Mirek Laus (mirek@laus.cz)

from bases.FrameworkServices.SocketService import SocketService

ORDER = [
    'Queue',
    'Events',
    'Tree',
    'Journal'
]

CHARTS = {
    'Queue': {
        'options': [None, 'rrdcached queue length', 'items', 'queue', 'rrdcached.queue', 'line'],
        'lines': [
            ['QueueLength', 'queue length', 'absolute'],
            ['UpdatesWritten', 'updates written', 'incremental'],
            ['FlushesReceived', 'flushes received', 'incremental']
        ]
    },
    'Events': {
        'options': [None, 'rrdcached updates', 'events/s', 'events', 'rrdcached.events', 'line'],
        'lines': [
            ['UpdatesReceived', 'updates received', 'incremental'],
            ['DataSetsWritten', 'datasets written', 'incremental']
        ]
    },
    'Tree': {
        'options': [None, 'rrdcached tree', 'items', 'tree', 'rrdcached.tree', 'line'],
        'lines': [
            ['TreeNodesNumber', 'nodes number', 'absolute'],
            ['TreeDepth', 'tree depth', 'absolute']
        ]
    },
    'Journal': {
        'options': [None, 'rrdcached journal', 'size', 'journal', 'rrdcached.journal', 'line'],
        'lines': [
            ['JournalBytes', 'journal size', 'absolute', 1, 1024*1024*1024],
            ['JournalRotate', 'journal rotate', 'absolute']
        ]
    }
}


class Service(SocketService):
    def __init__(self, configuration=None, name=None):
        SocketService.__init__(self, configuration=configuration, name=name)
        self.order = ORDER
        self.definitions = CHARTS
        self.host = self.configuration.get('host', '127.0.0.1')
        self.port = self.configuration.get('port', 42217)
        self.unix_socket = self.configuration.get('socket')
        self.request = 'STATS\r\n'

    def _get_data(self):
        """
        Format data received from socket
        :return: dict
        """
        try:
            raw = self._get_raw_data()
        except (ValueError, AttributeError):
            return None

        if raw is None:
            self.debug('rrdcached returned no data')
            return None

        lines = raw.split('\n')
        ret = dict()
        for i, line in enumerate(lines):
            try:
                data = line.split(': ')[:2]
                ret[str(data[0])] = int(data[1])
            except (ValueError, IndexError) as e:
                continue
        return ret or None
